#ifndef _ANOVA_H_
#define _ANOVA_H_
#define COMMENT_STRING         "Perform ANOVA on a DOE and a result vector"
#define COMMENT_PROGRAM_NAME   "ANOVA Calculator w/ Pooling"
#define COMMENT_VERSION_STRING "v5.2"
#define COMMENT_VERSION_NUMBER 5,2,0,0 /* major, minor, revision, subrevision */
#include <tigcclib.h>
#include "mlib.h"

#define RVEC_LOC 200			// The horizontal location of the result vector output (Voyage 200 LCD is 240x128)
#define DEFAULT_LBLLEN 15		// Default length limit of factor column labels
ESI argptr, tptr;				// Stack pointers
const float epsilon = 0.000001;	// a float within epsilon of 0 is considered to be 0 (you can't check x==0.0 if x is  a float)
// This constants are set as global literals for max efficiency, so that we aren't repeating
// heavily used literals within a scope many times.  This is what happens with a when you do
// '#define XSTRING "some string"' and then use XSTRING in, say, a loop.  That literal causes
// a memory allocation each time it is encountered.
char  DASH[3] = "C-", 			// Dash for empty cells in a table (centered)
	POOLSTR[8] = "CPOOLED",		// Label for variances for pooled variables (centered)
	ERRSTR[4] = "ERR",			// Error line label
	NASTR[5] = "Cn/a",			// For Ve when undefined
	PROMPT[] = "Press a key to continue.",	// Default prompt
	SPACES[] = "                                          ";	// Used to erase the prompt
#define USEFONT F_4x6			// The default font
#define FONTHEIGHT (ushort)6	// The dimensions of the default font
#define FONTWIDTH (ushort)4

// First, there are the dimensions of the ANOVA result table
ushort pos[7] = {0, 22, 38, 78, 118, 158, 198},			// horizontal pixel coords of the columns
	widths[7] = {5,3,9,9,9,9,8};							// column widths (in characters)
char lbls[7][5] = {"Fac","f","S","V","F","S'","P(%)"};	// column labels
// Shared between printANOVA() and fmtANOVA()
Matrix **mlist;
float **footer;

#define Pause(p) ({SCR_STATE ss; SaveScrState(&ss); puts(p); ngetchx(); MoveTo(ss.CurX,ss.CurY); puts(SPACES); MoveTo(ss.CurX,ss.CurY);})

typedef struct {				// ANOVA variables
  Matrix *factors, *results;	// factor matrix (mxn) & result vec (mx1)
  Matrix *SSf;					// Sum squares of factors					err(186)/spc(465)
  Matrix *fdof;					// Factor degrees of freedom				err(192)/spc(464)
  Matrix *Vf;					// Variance of factors						err(208)/spc(200)
  Matrix *F;					// Factor F-ratios							err()/spc(233)
  Matrix *SSfp;					// Pure sum squares of factors				err()/spc(234)
  Matrix *Pf;					// Relative contributions of factors		err(215)/spc(466)
  float T, Cf,					// Total of results and correction factor
	SSt, SSe,					// Sums of squares (total, error)
	ft, fe,						// DOF's (total, error)
	Ve, Pe;						// Variance and % contrib of error
  ushort levels;				// Levels (assumed the same for all factors)
  Matrix *lvlsum, *lvlcnt;		// Level sum and count matrices (levels-by-n)
  char **clbls;					// Column labels
  ushort *poolVars;				// column numbers of variables to pool, with 0 to mark the end
} ANVSet;

void usage();							// Print usage message and exit
void InitANVSet(ANVSet **set);			// Initialize the dataset
ANVSet *ANVSetup();						// Setup the environment and screen
void GetArgs(ANVSet *data);				// Get the program arguments (and do debug mode output)
void calcANOVA(ANVSet *data);			// Perform ANOVA
void printANOVA(ANVSet *data);			// Print the ANOVA table to the screen
int askOptions(char **, ANVSet *);		// Ask the user what to do next
void fmtANOVA(ANVSet *data);			// Format the result data and push it onto the stack
void outputANOVA(char *outfn);			// Dispose of output data accordingly
void askPooling(ANVSet *data);			// Ask the user if pooling should be done
void calcAPool(ANVSet *data);			// Perform ANOVA w/ pooled variables
void showInputs(ANVSet *data);			// Show inputs supplied to program
#endif